@@ -6,22 +6,19 @@
 //!
 //! ```no_run
 //! use leaky_bucket_lite::sync_threadsafe::LeakyBucket;
-//! use std::{error::Error, time::Duration};
+//! use std::time::Duration;
 //!
-//! fn main() -> Result<(), Box<dyn Error>> {
-//!     let rate_limiter = LeakyBucket::builder()
-//!         .max(5.0)
-//!         .tokens(0.0)
-//!         .refill_interval(Duration::from_secs(1))
-//!         .refill_amount(1.0)
-//!         .build();
+//! let rate_limiter = LeakyBucket::builder()
+//!     .max(5)
+//!     .tokens(0)
+//!     .refill_interval(Duration::from_secs(1))
+//!     .refill_amount(1)
+//!     .build();
 //!
-//!     println!("Waiting for permit...");
-//!     // should take about 5 seconds to acquire.
-//!     rate_limiter.acquire(5.0);
-//!     println!("I made it!");
-//!     Ok(())
-//! }
+//! println!("Waiting for permit...");
+//! // should take about 5 seconds to acquire.
+//! rate_limiter.acquire(5);
+//! println!("I made it!");
 //! ```
 #[cfg(feature = "parking_lot")]
 use parking_lot::{Mutex, MutexGuard};
@@ -35,21 +32,21 @@ use std::{
 /// The mutable internals of the leaky bucket.
 #[derive(Debug)]
 struct LeakyBucketInner {
-    tokens: f64,
+    tokens: u32,
     last_refill: Instant,
 }
 
 /// The leaky bucket.
 #[derive(Clone, Debug)]
 pub struct LeakyBucket {
-    max: f64,
+    max: u32,
     refill_interval: Duration,
-    refill_amount: f64,
+    refill_amount: u32,
     inner: Arc<Mutex<LeakyBucketInner>>,
 }
 
 impl LeakyBucket {
-    fn new(max: f64, tokens: f64, refill_interval: Duration, refill_amount: f64) -> Self {
+    fn new(max: u32, tokens: u32, refill_interval: Duration, refill_amount: u32) -> Self {
         Self {
             max,
             refill_interval,
@@ -64,14 +61,15 @@ impl LeakyBucket {
     #[inline]
     fn update_tokens(&self, inner: &mut MutexGuard<'_, LeakyBucketInner>) {
         let time_passed = Instant::now() - inner.last_refill;
-        let refills_since =
-            (time_passed.as_secs_f64() / self.refill_interval.as_secs_f64()).floor();
-        inner.tokens += self.refill_amount * refills_since;
-        inner.last_refill += self.refill_interval.mul_f64(refills_since);
 
-        if inner.tokens > self.max {
-            inner.tokens = self.max;
-        }
+        #[allow(clippy::cast_possible_truncation, clippy::cast_sign_loss)]
+        let refills_since =
+            (time_passed.as_secs_f64() / self.refill_interval.as_secs_f64()).floor() as u32;
+
+        inner.tokens += self.refill_amount * refills_since;
+        inner.last_refill += self.refill_interval * refills_since;
+
+        inner.tokens = inner.tokens.min(self.max);
     }
 
     /// Construct a new leaky bucket through a builder.
@@ -82,13 +80,13 @@ impl LeakyBucket {
 
     /// Get the max number of tokens this rate limiter is configured for.
     #[must_use]
-    pub const fn max(&self) -> f64 {
+    pub const fn max(&self) -> u32 {
         self.max
     }
 
     /// Get the current number of tokens available.
     #[must_use]
-    pub fn tokens(&self) -> f64 {
+    pub fn tokens(&self) -> u32 {
         #[cfg(not(feature = "parking_lot"))]
         let mut inner = self.inner.lock().expect("Mutex poisoned");
         #[cfg(feature = "parking_lot")]
@@ -107,31 +105,23 @@ impl LeakyBucket {
     ///
     /// ```rust
     /// use leaky_bucket_lite::sync_threadsafe::LeakyBucket;
-    /// use std::{error::Error, time::Duration};
+    /// use std::time::Duration;
     ///
-    /// fn main() -> Result<(), Box<dyn Error>> {
-    ///     let rate_limiter = LeakyBucket::builder()
-    ///         .max(5.0)
-    ///         .tokens(0.0)
-    ///         .refill_interval(Duration::from_secs(5))
-    ///         .refill_amount(1.0)
-    ///         .build();
+    /// let rate_limiter = LeakyBucket::builder()
+    ///     .max(5)
+    ///     .tokens(0)
+    ///     .refill_interval(Duration::from_secs(5))
+    ///     .refill_amount(1)
+    ///     .build();
     ///
-    ///     println!("Waiting for permit...");
-    ///     // should take about 5 seconds to acquire.
-    ///     rate_limiter.acquire_one();
-    ///     println!("I made it!");
-    ///
-    ///     Ok(())
-    /// }
+    /// println!("Waiting for permit...");
+    /// // should take about 5 seconds to acquire.
+    /// rate_limiter.acquire_one();
+    /// println!("I made it!");
     /// ```
-    ///
-    /// # Errors
-    ///
-    /// Returns an `Error` when communicating with the actor fails.
     #[inline]
     pub fn acquire_one(&self) {
-        self.acquire(1.0);
+        self.acquire(1);
     }
 
     /// Acquire the given `amount` of tokens.
@@ -140,29 +130,31 @@ impl LeakyBucket {
     ///
     /// ```rust
     /// use leaky_bucket_lite::sync_threadsafe::LeakyBucket;
-    /// use std::{error::Error, time::Duration};
+    /// use std::time::Duration;
     ///
-    /// fn main() -> Result<(), Box<dyn Error>> {
-    ///     let rate_limiter = LeakyBucket::builder()
-    ///         .max(5.0)
-    ///         .tokens(0.0)
-    ///         .refill_interval(Duration::from_secs(5))
-    ///         .refill_amount(1.0)
-    ///         .build();
+    /// let rate_limiter = LeakyBucket::builder()
+    ///     .max(5)
+    ///     .tokens(0)
+    ///     .refill_interval(Duration::from_secs(5))
+    ///     .refill_amount(1)
+    ///     .build();
     ///
-    ///     println!("Waiting for permit...");
-    ///     // should take about 25 seconds to acquire.
-    ///     rate_limiter.acquire(5.0);
-    ///     println!("I made it!");
-    ///
-    ///     Ok(())
-    /// }
+    /// println!("Waiting for permit...");
+    /// // should take about 25 seconds to acquire.
+    /// rate_limiter.acquire(5);
+    /// println!("I made it!");
     /// ```
     ///
-    /// # Errors
+    /// # Panics
     ///
-    /// Returns an `Error` when communicating with the actor fails.
-    pub fn acquire(&self, amount: f64) {
+    /// This method will panic when acquiring more tokens than the configured maximum.
+
+    pub fn acquire(&self, amount: u32) {
+        assert!(
+            amount <= self.max(),
+            "Acquiring more tokens than the configured maximum is not possible"
+        );
+
         #[cfg(not(feature = "parking_lot"))]
         let mut inner = self.inner.lock().expect("Mutex poisoned");
         #[cfg(feature = "parking_lot")]
@@ -172,8 +164,14 @@ impl LeakyBucket {
 
         if inner.tokens < amount {
             let tokens_needed = amount - inner.tokens;
-            let refills_needed = (tokens_needed / self.refill_amount).ceil();
-            let target_time = inner.last_refill + self.refill_interval.mul_f64(refills_needed);
+            let mut refills_needed = tokens_needed / self.refill_amount;
+            let refills_needed_remainder = tokens_needed % self.refill_amount;
+
+            if refills_needed_remainder > 0 {
+                refills_needed += 1;
+            }
+
+            let target_time = inner.last_refill + self.refill_interval * refills_needed;
 
             std::thread::sleep(target_time - Instant::now());
 
@@ -187,10 +185,10 @@ impl LeakyBucket {
 /// Builder for a leaky bucket.
 #[derive(Debug)]
 pub struct Builder {
-    max: Option<f64>,
-    tokens: Option<f64>,
+    max: Option<u32>,
+    tokens: Option<u32>,
     refill_interval: Option<Duration>,
-    refill_amount: Option<f64>,
+    refill_amount: Option<u32>,
 }
 
 impl Builder {
@@ -207,7 +205,7 @@ impl Builder {
 
     /// Set the max value for the builder.
     #[must_use]
-    pub const fn max(mut self, max: f64) -> Self {
+    pub const fn max(mut self, max: u32) -> Self {
         self.max = Some(max);
         self
     }
@@ -216,7 +214,7 @@ impl Builder {
     ///
     /// If set to larger than `max` at build time, will only saturate to max.
     #[must_use]
-    pub const fn tokens(mut self, tokens: f64) -> Self {
+    pub const fn tokens(mut self, tokens: u32) -> Self {
         self.tokens = Some(tokens);
         self
     }
@@ -230,7 +228,7 @@ impl Builder {
 
     /// Set the refill amount to use.
     #[must_use]
-    pub const fn refill_amount(mut self, refill_amount: f64) -> Self {
+    pub const fn refill_amount(mut self, refill_amount: u32) -> Self {
         self.refill_amount = Some(refill_amount);
         self
     }
@@ -238,10 +236,10 @@ impl Builder {
     /// Construct a new leaky bucket.
     #[must_use]
     pub fn build(self) -> LeakyBucket {
-        const DEFAULT_MAX: f64 = 120.0;
-        const DEFAULT_TOKENS: f64 = 0.0;
+        const DEFAULT_MAX: u32 = 120;
+        const DEFAULT_TOKENS: u32 = 0;
         const DEFAULT_REFILL_INTERVAL: Duration = Duration::from_secs(1);
-        const DEFAULT_REFILL_AMOUNT: f64 = 1.0;
+        const DEFAULT_REFILL_AMOUNT: u32 = 1;
 
         let max = self.max.unwrap_or(DEFAULT_MAX);
         let tokens = self.tokens.unwrap_or(DEFAULT_TOKENS);
